
/*
Tastatur und Display für Linearanschlag - Zabex 2016


Keypad Bibliothek importieren über: 
Sketch -> Library importieren -> Add Library…


Pin:  2..9 Keyboard
      10..12 7-Segment display
*/

/*4x4 Matrix Keypad connected to Arduino
 This code prints the key pressed on the keypad to the serial port

 Received data is displayed on 8 Digit 7Segment display (MAX7219 controller)
  
 */
 
#include <Keypad.h>
#include "max7219.h"
 
const byte numRows= 4; //number of rows on the keypad
const byte numCols= 4; //number of columns on the keypad
 
//keymap defines the key pressed according to the row and columns just as appears on the keypad
char keymap[numRows][numCols]= 
{
  {'1', '2', '3', 'A'}, 
  {'4', '5', '6', 'B'}, 
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
};
 
//Code that shows the the keypad connections to the arduino terminals
byte rowPins[numRows] = {9,8,7,6}; //Rows 0 to 3
byte colPins[numCols]= {5,4,3,2}; //Columns 0 to 3
byte index=0;
int  val=0; 
bool dot=false;
int rxIndex=8;
//initializes an instance of the Keypad class
Keypad myKeypad= Keypad(makeKeymap(keymap), rowPins, colPins, numRows, numCols);

/*
 Led controller MAX7219:
 pin 12 is connected to the DataIn 
 pin 10 is connected to the CLK 
 pin 11 is connected to the CS 
 */
// initialize the library with the numbers of the interface pins
MAX7219 led=MAX7219(12,10,11);
 
//--------------------------------------------------------------------------------------------------- 
void setup()
//--------------------------------------------------------------------------------------------------- 
{
    Serial.begin(9600);
    //The led controller MAX7219 is in power-saving mode on startup. Make him run.
    led.shutdown(false);
    /* Set the brightness(0..15) to a low value */
    led.setIntensity(4);
    led.clearDisplay(); 
    for (byte i=0; i<8; i++){
      led.setChar(i, '-', false);
    }
    //Serial.setTimeout(800);
    Serial.println("@");
}

//--------------------------------------------------------------------------------------------------- 
//If key is pressed, then this key is printed out
//Debouncing and capture "Button Down" event is done already in the library. 
//--------------------------------------------------------------------------------------------------- 
void loop()
//--------------------------------------------------------------------------------------------------- 
{
  char keypressed = myKeypad.getKey();
  if (keypressed != NO_KEY) {
      Serial.write(keypressed);
      Serial.println("");
      Serial.flush();
  }
}

/*
  SerialEvent occurs whenever a new data comes in the
 hardware serial RX.  This routine is run between each
 time loop() runs, so using delay inside loop can delay
 response.  Multiple bytes of data may be available.
 */
void serialEvent() {
    while (Serial.available()) {
        // get the new byte:
        char inChar = (char)Serial.read();
        if ('.' == inChar) { //cls, Alle stellen beschreiben.
          dot = true;
        }else
        if ('*' == inChar) { //cls, Alle stellen beschreiben.
          led.clearDisplay();
          dot = false;
          rxIndex = 7;
        }else
        if ('+' == inChar) { // 4 rechten Stellen beschreiben
          dot = false;
          rxIndex = 3;
        }else
        if (inChar >=' ') { //Zeichen schreiben
           led.setChar(rxIndex, inChar, dot);
           if (rxIndex > 0) rxIndex--;
           dot = false;
        }  
    }//while
}
